package com.bsf.lovDemo.components;

import org.bsf.listOfValues.client.LovManager;
import org.bsf.listOfValues.exceptions.NoSuchLovException;
import org.bsf.listOfValues.exceptions.NoSuchLovValueException;
import org.bsf.listOfValues.lovValue.LovValue;

import javax.swing.JTable;
import javax.swing.table.DefaultTableCellRenderer;
import java.awt.Color;
import java.awt.Component;
import java.rmi.RemoteException;

/**
 * Basic renderer for a cell containing a LOV. It search, in the LOV that it is
 * set to use, the LovValue matching the value (ie. OID) given and returns a
 * component containing the field it was set to use instead of the OID itself.
 */
public class LovCellRenderer extends DefaultTableCellRenderer {
    private static String EMPTY_STRING = "";
    private static final Color LOV_FG_COLOR = new Color( 9, 63, 233 );

    private Long _lovOID = null;
    private String _fieldName = null;
    private String _tooltipName = null;

    // *************************************************************************
    // ****************************** Constructor ******************************
    // *************************************************************************

    /**
     * Instantiate a LovCellRenderer that will use the given OID to retrieve the
     * LOV and will display the value corresponding to the given fieldName.
     *
     * @param p_lovOID The oid of the LOV to use to check for the LovValue.
     * @param p_fieldName The name of the field to use for display.
     * @param p_tooltipFieldName The name of the field to use for the tooltip, it
     * can be null.
     *
     * @throws IllegalArgumentException if the lovOId and/or the fieldName
     * is/are null.
     */
    public LovCellRenderer( Long p_lovOID, String p_fieldName, String p_tooltipFieldName ) {
        super();

        if ( p_lovOID == null || p_fieldName == null ) {
            throw new IllegalArgumentException( "Need a non null lovOID and fieldName..." );
        }

        _lovOID = p_lovOID;
        _fieldName = p_fieldName;
        _tooltipName = p_tooltipFieldName;
    }

    // *************************************************************************
    // ************* Overiden Method from DefaultTableCellRenderer *************
    // *************************************************************************

    /**
     * This method is replacing the LovValueOID by the value of the field
     * contained in the corresponding LovValue (cf. header). It could be
     * greatly improved to be fatser but this is just to illustrate how
     * to use the lov.
     *
     * @return
     */
    public Component getTableCellRendererComponent( JTable p_table, Object p_value,
                                                    boolean p_isSelected, boolean p_hasFocus,
                                                    int p_row, int p_column ) {

        // We want the default rendering (no time to make one from scratch :o)
        super.getTableCellRendererComponent( p_table, p_value, p_isSelected, p_hasFocus, p_row, p_column );

        // But we might have to replace using the field found in the matching LovValue
        if ( p_value instanceof Long ) { // Must be an OID (ie. a Long)
            try {
                LovValue lovValue = LovManager.getLovValue( _lovOID, (Long) p_value );

                // We find and set the requested field value in the label
                Object fielValue = lovValue.getFieldValue( _fieldName );
                Object tooltip = lovValue.getFieldValue( _tooltipName );

                // On a perfect LovRenderer you should handle the type of the Object
                setText( fielValue != null ? fielValue.toString() : EMPTY_STRING );
                setToolTipText( tooltip != null ? tooltip.toString() : null );
            } catch( NoSuchLovValueException e ) {
                // Here you should handle the fact that there's no LovValue with
                // an oid matching the given p_value !!!
                setText( "The request Lov (" + _lovOID + ") has no LovValue matching the oid " + p_value );
            } catch( NoSuchLovException e ) {
                // Here you should handle the fact that the selected LOV does not exist !!!
                // For the demo we just display a message stating that the lov doesn't exist
                setText( "The request Lov (" + _lovOID + ") isn't defined " );
            } catch( RemoteException e ) {
                // A network problem... We do nothing... We did what we could...
                // We leave the oid in the label...
            } finally {
                // We change the foreground color if the cell ain't selected
                if ( !p_isSelected ) {
                    setForeground( LOV_FG_COLOR );
                }
            }
        }

        return this;
    }

    /**
     * Enable to change the rendering field.
     *
     * @param p_fieldName The field to use for the rendering.
     */
    public void setDisplayedField( String p_fieldName ) {
        _fieldName = p_fieldName;
    }
}
