/**
 * Copyright (c) 2002 Bright Side Factory.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Bright Side Factory (http://www.bs-factory.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Bright Side", "BS Factory" and "Bright Side Factory" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact info@bs-factory.org.
 *
 * 5. Products derived from this software may not be called "Bright Side",
 *    nor may "Bright Side" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Bright Side Factory.  For more
 * information on the Bright Side Factory, please see
 * <http://www.bs-factory.org/>.
 */

package com.bsf.lovDemo.controllers;

import com.bsf.lovDemo.views.DemoFrame;
import com.bsf.lovDemo.views.MainPage;
import com.jgoodies.swing.application.ActionManager;
import com.jgoodies.swing.application.Workbench;

import javax.swing.AbstractAction;
import javax.swing.Action;
import java.awt.event.ActionEvent;

/**
 * Provides all UI <code>Actions</code> and their IDs.
 * Therefore it declares static fields for action ids
 * and implementations of the <code>Action</code> interface.
 * These actions are registered to the <code>ActionManager</code>.
 * <p>
 * This class demos three different styles to implement <code>Action</code>;
 * they differ in readability, number of classes loaded, and the ability
 * to write more or less large action bodies, while preserving good
 * code formatting:
 * <ol>
 * <li>anonymous classes: This approach provides a literally compact way
 * to write down a bunch of action implementations, which will work better,
 * if you have tiny method bodys. In our demo, all method bodies just
 * delegate to the application controllers. Does not provide human readable
 * name in case of an error and so, may be more difficult to debug.
 * Requires a class for every action.
 * <li>nested top-level classes: This approach requires a little bit
 * more writing than the first approach. Writing larger method bodies
 * does not affect the registering method. The extra class names
 * help you find bugs.
 * Requires a class for every action.
 * <li>instances of a single dispatching class: This approach leads
 * to good readability in the registering method and requires only
 * one class to load for all actions. Works better with small action bodies.
 * May lead to runtime errors, if you haven't defined a dispatch for
 * an action ID.
 * </ol>
 *
 * @see     Action
 * @see     ActionManager
 * @see     AbstractAction
 */

public final class Actions {
    // Custom actions
    public static final String SHOW_WELCOME = "showWelcome";
    public static final String SHOW_LOV_IN_TABLE = "showLovInTable";
    public static final String SHOW_LOV_COMBOBOX = "showLovInComboBox";
    public static final String SHOW_LOV_REQUEST_SETUP = "showLovRequestSetup";
    public static final String SHOW_LOV_PERFORMANCES = "showLovPerformances";

    // Standard actions
    public static final String EXIT_ID = "exit";

    /**
     * Registers actions in the <code>ActionManager</code> using
     * a bunch of anonymous classes.
     */
    public static void registerActions() {
        ActionManager.register( EXIT_ID, new AbstractAction() {
            public void actionPerformed( ActionEvent event ) {
                Workbench.getMainFrame().aboutToExitApplication();
            }
        } );

        ActionManager.register( SHOW_WELCOME, new AbstractAction() {
            public void actionPerformed( ActionEvent event ) {
                ( (DemoFrame) Workbench.getMainFrame() ).setDisplayedInternalFrame( MainPage.WELCOME_INTERNAL_FRAME );
            }
        } );

        ActionManager.register( SHOW_LOV_IN_TABLE, new AbstractAction() {
            public void actionPerformed( ActionEvent event ) {
                ( (DemoFrame) Workbench.getMainFrame() ).setDisplayedInternalFrame( MainPage.LOV_IN_TABLE_INTERNAL_FRAME );
            }
        } );

        ActionManager.register( SHOW_LOV_COMBOBOX, new AbstractAction() {
            public void actionPerformed( ActionEvent event ) {
                ( (DemoFrame) Workbench.getMainFrame() ).setDisplayedInternalFrame( MainPage.LOV_IN_DETAIL_INTERNAL_FRAME );
            }
        } );

        ActionManager.register( SHOW_LOV_REQUEST_SETUP, new AbstractAction() {
            public void actionPerformed( ActionEvent event ) {
                ( (DemoFrame) Workbench.getMainFrame() ).setDisplayedInternalFrame( MainPage.LOV_CUSTOMIZATION_INTERNAL_FRAME );
            }
        } );

        ActionManager.register( SHOW_LOV_PERFORMANCES, new AbstractAction() {
            public void actionPerformed( ActionEvent event ) {
                ( (DemoFrame) Workbench.getMainFrame() ).setDisplayedInternalFrame( MainPage.LOV_PERFORMANCES_INTERNAL_FRAME );
            }
        } );
    }
}
